#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// A demonstration indicator that retreives an SMA value from the previous day/session
    /// </summary>
    [Description("A demonstration indicator that retreives an SMA value from the previous day/session")]
    public class SampleGetBar : Indicator
    {
        #region Variables
        private int 		sMAPeriod	= 14; // Default setting for sMAPeriod
		private int			barsAgo 	= 0;  // Variable to hold a bars ago value.
		private DateTime	timeOfInterest;   // DateTime object to hold the time we're interested in.
        #endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
            Add(new Plot(Color.FromKnownColor(KnownColor.Orange), PlotStyle.Line, "SMAPlot"));
            CalculateOnBarClose	= true;
            Overlay				= true;
            PriceTypeSupported	= false;
        }

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
			// Plots the SMA indicator.
			SMAPlot.Set(SMA(sMAPeriod)[0]);
			
			/* This sample assumes instrument sessions that go from Monday-Friday and not the weekends.
			   If you are using an instrument that has weekend trading sessions like the ES futures, please be aware that the logic presented will need to be
			   modified to reflect the trading sessions for your particular instrument.
			   Here, timeOfInterest is reset once per day at session break. */
			if (Bars.FirstBarOfSession && FirstTickOfBar)
			{
				/* If the day is Monday, and we want the value from Friday, we must subtract 3 days from the current date.
				   If the day is Tuesday-Friday, just subtract one day. */
				if (Time[0].DayOfWeek == DayOfWeek.Monday)
					timeOfInterest = new DateTime(Time[0].Year, Time[0].Month, Time[0].Day - 3, 9, 30, 0);
				else
					timeOfInterest = new DateTime(Time[0].Year, Time[0].Month, Time[0].Day - 1, 9, 30, 0);
			}
			
			/* Determine the number of bars ago it would take to access the 9:30AM bar for the previous trading day.
			   GetBar() returns either the bars ago number for the 9:30AM bar if it exists or the number for the first bar after 9:30AM 
			   if there isn’t a specific bar with the exact 9:30AM timestamp (e.g. tick-based bars). With tick bars, it is possible 
			   for many bars to have the same exact time stamp. In this case, the first bar of all the bars with the same timestamp is returned.*/
			barsAgo = GetBar(timeOfInterest);
			
			/* If GetBar returns 0, the time you are looking for would either be the current bar or at some time after the current bar.
			   Also, make sure there are enough bars to draw the line 10 bars back. */
			if (barsAgo != 0 && CurrentBar > 10)
			{
				// Draw a line with the value of the SMA the correct number of bars ago.
				DrawLine("yesterday SMA", false, 10, SMAPlot[barsAgo], 0, SMAPlot[barsAgo], Color.Black, DashStyle.Solid, 2);
            }
        }

        #region Properties
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries SMAPlot
        {
            get { return Values[0]; }
        }

        [Description("")]
        [Category("Parameters")]
        public int SMAPeriod
        {
            get { return sMAPeriod; }
            set { sMAPeriod = Math.Max(1, value); }
        }
        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private SampleGetBar[] cacheSampleGetBar = null;

        private static SampleGetBar checkSampleGetBar = new SampleGetBar();

        /// <summary>
        /// A demonstration indicator that retreives an SMA value from the previous day/session
        /// </summary>
        /// <returns></returns>
        public SampleGetBar SampleGetBar(int sMAPeriod)
        {
            return SampleGetBar(Input, sMAPeriod);
        }

        /// <summary>
        /// A demonstration indicator that retreives an SMA value from the previous day/session
        /// </summary>
        /// <returns></returns>
        public SampleGetBar SampleGetBar(Data.IDataSeries input, int sMAPeriod)
        {
            if (cacheSampleGetBar != null)
                for (int idx = 0; idx < cacheSampleGetBar.Length; idx++)
                    if (cacheSampleGetBar[idx].SMAPeriod == sMAPeriod && cacheSampleGetBar[idx].EqualsInput(input))
                        return cacheSampleGetBar[idx];

            lock (checkSampleGetBar)
            {
                checkSampleGetBar.SMAPeriod = sMAPeriod;
                sMAPeriod = checkSampleGetBar.SMAPeriod;

                if (cacheSampleGetBar != null)
                    for (int idx = 0; idx < cacheSampleGetBar.Length; idx++)
                        if (cacheSampleGetBar[idx].SMAPeriod == sMAPeriod && cacheSampleGetBar[idx].EqualsInput(input))
                            return cacheSampleGetBar[idx];

                SampleGetBar indicator = new SampleGetBar();
                indicator.BarsRequired = BarsRequired;
                indicator.CalculateOnBarClose = CalculateOnBarClose;
#if NT7
                indicator.ForceMaximumBarsLookBack256 = ForceMaximumBarsLookBack256;
                indicator.MaximumBarsLookBack = MaximumBarsLookBack;
#endif
                indicator.Input = input;
                indicator.SMAPeriod = sMAPeriod;
                Indicators.Add(indicator);
                indicator.SetUp();

                SampleGetBar[] tmp = new SampleGetBar[cacheSampleGetBar == null ? 1 : cacheSampleGetBar.Length + 1];
                if (cacheSampleGetBar != null)
                    cacheSampleGetBar.CopyTo(tmp, 0);
                tmp[tmp.Length - 1] = indicator;
                cacheSampleGetBar = tmp;
                return indicator;
            }
        }
    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// A demonstration indicator that retreives an SMA value from the previous day/session
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.SampleGetBar SampleGetBar(int sMAPeriod)
        {
            return _indicator.SampleGetBar(Input, sMAPeriod);
        }

        /// <summary>
        /// A demonstration indicator that retreives an SMA value from the previous day/session
        /// </summary>
        /// <returns></returns>
        public Indicator.SampleGetBar SampleGetBar(Data.IDataSeries input, int sMAPeriod)
        {
            return _indicator.SampleGetBar(input, sMAPeriod);
        }
    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// A demonstration indicator that retreives an SMA value from the previous day/session
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.SampleGetBar SampleGetBar(int sMAPeriod)
        {
            return _indicator.SampleGetBar(Input, sMAPeriod);
        }

        /// <summary>
        /// A demonstration indicator that retreives an SMA value from the previous day/session
        /// </summary>
        /// <returns></returns>
        public Indicator.SampleGetBar SampleGetBar(Data.IDataSeries input, int sMAPeriod)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.SampleGetBar(input, sMAPeriod);
        }
    }
}
#endregion
